@@ -20,29 +20,26 @@
 
 #include "slave.h"
 
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
+#include <qplatformdefs.h>
 #include <stdio.h>
-#include <signal.h>
-#include <sys/types.h>
 
 #include <QtCore/QFile>
 #include <QtCore/QTimer>
-#include <QDBusConnection>
 #include <QtCore/QProcess>
 
+#include <QDBusConnection>
 #include <klocalizedstring.h>
 
 #include <kdeinitinterface.h>
 #include <klauncher_interface.h>
-#include <klibrary.h>
+#include <kpluginloader.h>
 
 #include "dataprotocol_p.h"
 #include "kservice.h"
 #include "connection_p.h"
 #include "commands_p.h"
 #include "connectionserver.h"
+#include "kioglobal_p.h"
 #include <kprotocolinfo.h>
 #include <config-kiocore.h> // CMAKE_INSTALL_PREFIX
 
@@ -62,13 +59,19 @@ using namespace KIO;
 #define SLAVE_CONNECTION_TIMEOUT_MAX    3600
 #endif
 
-Q_GLOBAL_STATIC_WITH_ARGS(org::kde::KSlaveLauncher, klauncherIface,
-                          (QString::fromLatin1("org.kde.klauncher5"), QString::fromLatin1("/KLauncher"), QDBusConnection::sessionBus()))
+static QThreadStorage<org::kde::KSlaveLauncher *> s_kslaveLauncher;
 
 static org::kde::KSlaveLauncher *klauncher()
 {
     KDEInitInterface::ensureKdeinitRunning();
-    return ::klauncherIface();
+    if (!s_kslaveLauncher.hasLocalData()) {
+        org::kde::KSlaveLauncher *launcher = new org::kde::KSlaveLauncher(QString::fromLatin1("org.kde.klauncher5"),
+                QString::fromLatin1("/KLauncher"),
+                QDBusConnection::sessionBus());
+        s_kslaveLauncher.setLocalData(launcher);
+        return launcher;
+    }
+    return s_kslaveLauncher.localData();
 }
 
 namespace KIO
@@ -110,7 +113,7 @@ public:
     QString m_passwd;
     KIO::ConnectionServer *slaveconnserver;
     KIO::SimpleJob *m_job;
-    pid_t m_pid;
+    qint64 m_pid;
     quint16 m_port;
     bool contacted;
     bool dead;
@@ -142,7 +145,7 @@ void Slave::timeout()
 
     /*qDebug() << "slave failed to connect to application pid=" << d->m_pid
                  << " protocol=" << d->m_protocol;*/
-    if (d->m_pid && (::kill(d->m_pid, 0) == 0)) {
+    if (d->m_pid && KIOPrivate::isProcessAlive(d->m_pid)) {
         int delta_t = d->contact_started.secsTo(QDateTime::currentDateTime());
         //qDebug() << "slave is slow... pid=" << d->m_pid << " t=" << delta_t;
         if (delta_t < SLAVE_CONNECTION_TIMEOUT_MAX) {
@@ -269,13 +272,13 @@ int Slave::idleTime()
     return d->m_idleSince.secsTo(QDateTime::currentDateTime());
 }
 
-void Slave::setPID(pid_t pid)
+void Slave::setPID(qint64 pid)
 {
     Q_D(Slave);
     d->m_pid = pid;
 }
 
-int Slave::slave_pid()
+qint64 Slave::slave_pid()
 {
     Q_D(Slave);
     return d->m_pid;
@@ -377,11 +380,7 @@ void Slave::kill()
     /*qDebug() << "killing slave pid" << d->m_pid
                  << "(" << QString(d->m_protocol) + "://" + d->m_host << ")";*/
     if (d->m_pid) {
-#ifndef _WIN32_WCE
-        ::kill(d->m_pid, SIGTERM);
-#else
-        ::kill(d->m_pid, SIGKILL);
-#endif
+        KIOPrivate::sendTerminateSignal(d->m_pid);
         d->m_pid = 0;
     }
 }
@@ -432,17 +431,23 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
     // In such case we start the slave via QProcess.
     // It's possible to force this by setting the env. variable
     // KDE_FORK_SLAVES, Clearcase seems to require this.
-    static bool bForkSlaves = !qgetenv("KDE_FORK_SLAVES").isEmpty();
+    static QBasicAtomicInt bForkSlaves = Q_BASIC_ATOMIC_INITIALIZER(-1);
 
-    if (!bForkSlaves) {
-        // check the UID of klauncher
-        QDBusReply<uint> reply = QDBusConnection::sessionBus().interface()->serviceUid(klauncher()->service());
-        if (reply.isValid() && getuid() != reply) {
-            bForkSlaves = true;
+    if (bForkSlaves.load() == -1) {
+        bool fork = !qgetenv("KDE_FORK_SLAVES").isEmpty();
+
+        if (!fork) {
+            // check the UID of klauncher
+            QDBusReply<uint> reply = QDBusConnection::sessionBus().interface()->serviceUid(klauncher()->service());
+            if (reply.isValid() && getuid() != reply) {
+                fork = true;
+            }
         }
+
+        bForkSlaves.testAndSetRelaxed(-1, fork ? 1 : 0);
     }
 
-    if (bForkSlaves) {
+    if (bForkSlaves.load() == 1) {
         QString _name = KProtocolInfo::exec(protocol);
         if (_name.isEmpty()) {
             error_text = i18n("Unknown protocol '%1'.", protocol);
@@ -450,8 +455,10 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
             delete slave;
             return 0;
         }
-        KLibrary lib(_name);
-        QString lib_path = lib.fileName();
+        // find the kioslave using KPluginLoader; kioslave would do this
+        // anyway, but if it doesn't exist, we want to be able to return
+        // a useful error message immediately
+        QString lib_path = KPluginLoader::findPlugin(_name);
         if (lib_path.isEmpty()) {
             error_text = i18n("Can not find io-slave for protocol '%1'.", protocol);
             error = KIO::ERR_CANNOT_LAUNCH_PROCESS;
@@ -462,7 +469,7 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
         const QStringList args = QStringList() << lib_path << protocol << "" << slaveAddress.toString();
         //qDebug() << "kioslave" << ", " << lib_path << ", " << protocol << ", " << QString() << ", " << slaveAddress;
 
-        const QString kioslave = CMAKE_INSTALL_PREFIX "/" LIBEXEC_INSTALL_DIR "/kioslave";
+        const QString kioslave = CMAKE_INSTALL_PREFIX "/" KF5_LIBEXEC_INSTALL_DIR "/kioslave";
         if (kioslave.isEmpty()) {
             error_text = i18n("Can not find 'kioslave' executable");
             error = KIO::ERR_CANNOT_LAUNCH_PROCESS;
@@ -484,7 +491,7 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
         delete slave;
         return 0;
     }
-    pid_t pid = reply;
+    qint64 pid = reply;
     if (!pid) {
         error_text = i18n("Unable to create io-slave:\nklauncher said: %1", errorStr);
         error = KIO::ERR_CANNOT_LAUNCH_PROCESS;
@@ -510,7 +517,7 @@ Slave *Slave::holdSlave(const QString &protocol, const QUrl &url)
         delete slave;
         return 0;
     }
-    pid_t pid = reply;
+    qint64 pid = reply;
     if (!pid) {
         delete slave;
         return 0;
