@@ -717,10 +717,8 @@ QVariant KDirModel::data(const QModelIndex &index, int role) const
             case Name:
                 return item.text();
             case Size:
-                //
-                //return KIO::convertSize(item->size());
-                // Default to "file size in bytes" like in kde3's filedialog
-                return QLocale().toString(item.size());
+                // Filesize in bytes (type qulonglong)
+                return item.size();
             case ModifiedTime: {
                 QDateTime dt = item.time(KFileItem::ModificationTime);
                 return dt.toString();
@@ -900,7 +898,6 @@ int KDirModel::rowCount(const QModelIndex &parent) const
     return count;
 }
 
-// sibling() calls parent() and isn't virtual! So parent() should be fast...
 QModelIndex KDirModel::parent(const QModelIndex &index) const
 {
     if (!index.isValid()) {
@@ -913,6 +910,25 @@ QModelIndex KDirModel::parent(const QModelIndex &index) const
     return d->indexForNode(parentNode); // O(n)
 }
 
+// Reimplemented to avoid the default implementation which calls parent
+// (O(n) for finding the parent's row number for nothing). This implementation is O(1).
+QModelIndex KDirModel::sibling(int row, int column, const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return QModelIndex();
+    }
+    KDirModelNode *oldChildNode = static_cast<KDirModelNode *>(index.internalPointer());
+    Q_ASSERT(oldChildNode);
+    KDirModelNode *parentNode = oldChildNode->parent();
+    Q_ASSERT(parentNode);
+    Q_ASSERT(d->isDir(parentNode));
+    KDirModelNode *childNode = static_cast<KDirModelDirNode *>(parentNode)->m_childNodes.value(row); // O(1)
+    if (childNode) {
+        return createIndex(row, column, childNode);
+    }
+    return QModelIndex();
+}
+
 static bool lessThan(const QUrl &left, const QUrl &right)
 {
     return left.toString().compare(right.toString()) < 0;
@@ -993,16 +1009,6 @@ QMimeData *KDirModel::mimeData(const QModelIndexList &indexes) const
         data->setUrls(urls);
     }
 
-    // for compatibility reasons (when dropping or pasting into kde3 applications)
-    QString application_x_qiconlist;
-    const int items = urls.count();
-    for (int i = 0; i < items; i++) {
-        const int offset = i * 16;
-        QString tmp("%1$@@$%2$@@$32$@@$32$@@$%3$@@$%4$@@$32$@@$16$@@$no data$@@$");
-        application_x_qiconlist += tmp.arg(offset).arg(offset).arg(offset).arg(offset + 40);
-    }
-    data->setData("application/x-qiconlist", application_x_qiconlist.toLatin1());
-
     return data;
 }
 
